import { buildScaffoldOutput } from '../utils/generalUtils.js';
import { installPrettier, installSentry, installSentryForFrontend, } from '../utils/scaffoldAddOnUtils.js';
import { ConsoleLogger } from '../utils/consoleLogger.js';
import { LocaleManager } from '../cliHelpers/localeManager.js';
function buildDummyOutput() {
    const output = buildScaffoldOutput();
    output.message = 'Not implemented';
    return output;
}
/**
 * @deprecated
 * @description "object literal" to handle getting functions to install add ons
 * @param {STORMAddOn} stormAddOn
 */
export function scaffoldAddOns(stormAddOn) {
    const addOnInstallOpts = {
        prettier: async function () {
            return await installPrettier(process.cwd());
        },
        eslint: async function () {
            return buildDummyOutput();
        },
        storybook: async function () {
            return buildDummyOutput();
        },
        vitetest: async function () {
            return buildDummyOutput();
        },
        sentry: async function () {
            return await installSentry(process.cwd());
        },
    };
    return addOnInstallOpts[stormAddOn];
}
/**
 * @function scaffoldBackendAddons
 * @description Helper function that installs backend addons for the STORM stack
 * @param {STORMBEAddon} stormBEAddon the addon to be installed
 */
export function scaffoldBackendAddons(stormBEAddon) {
    const backendAddonInstallOpts = {
        sentry: async function () {
            return await installSentry(process.cwd());
        },
        vercel: async function () {
            return buildDummyOutput();
        },
    };
    return backendAddonInstallOpts[stormBEAddon];
}
/**
 * @function scaffoldFrontendAddons
 * @description Helper function that installs frontend addons for the STORM stack
 * @param {STORMFEAddon} stormFEAddon
 * @param {FrontendOpt} frontendOpt
 */
export function scaffoldFrontendAddons(stormFEAddon, frontendOpt) {
    const frontendAddonInstallOpts = {
        sentry: async function () {
            return installSentryForFrontend(process.cwd(), frontendOpt);
        },
        storybook: async function () {
            return buildDummyOutput();
        },
        vitest: async function () {
            return buildDummyOutput();
        },
    };
    return frontendAddonInstallOpts[stormFEAddon];
}
/**
 * @function scaffoldCodeQualityAddons
 * @description Helper function that installs code quality addons for the STORM stack
 * @param {STORMCodeQualityAddon} stormQCAddon
 */
export function scaffoldCodeQualityAddons(stormQCAddon) {
    const codeQualityAddonInstallOpts = {
        prettier: async function () {
            return await installPrettier(process.cwd());
        },
        eslint: async function () {
            return buildDummyOutput();
        },
        pycodestyle: async function () {
            return buildDummyOutput();
        },
    };
    return codeQualityAddonInstallOpts[stormQCAddon];
}
/**
 * @function installBEAddons
 * @description Handles installing backend addons into a STORM stack project
 * @param {Array<STORMBEAddon>} backendAddons
 * @param {LoggerMode} loggerMode
 */
export async function installBEAddons(backendAddons, loggerMode) {
    const verbose = loggerMode === 'verbose';
    for (const addon of backendAddons) {
        if (verbose)
            ConsoleLogger.printCLIProcessInfoMessage(`Installing Backend Addon`, addon);
        const addOnResult = await scaffoldBackendAddons(addon)();
        if (!addOnResult.success) {
            if (verbose)
                ConsoleLogger.printCLIProcessErrorMessage(`${addOnResult.message}`);
            process.exit(1);
        }
        if (verbose)
            ConsoleLogger.printCLIProcessSuccessMessage({
                message: 'Successfully installed addon',
            });
    }
}
/**
 * @function installFEAddons
 * @description Handles installing Frontend addons into a STORM stack project
 * @param {Array<STORMFEAddon>} frontendAddons
 * @param {FrontendOpt} frontendOpt
 * @param {LoggerMode} loggerMode
 */
export async function installFEAddons(frontendAddons, frontendOpt, loggerMode) {
    const verbose = loggerMode === 'verbose';
    for (const addon of frontendAddons) {
        if (verbose)
            ConsoleLogger.printCLIProcessInfoMessage(`Installing Frontend Addon`, addon);
        const addonResult = await scaffoldFrontendAddons(addon, frontendOpt)();
        if (!addonResult.success) {
            if (verbose) {
                ConsoleLogger.printCLIProcessErrorMessage(`${addonResult.message}`);
            }
            process.exit(1);
        }
        if (verbose)
            ConsoleLogger.printCLIProcessSuccessMessage({
                message: 'Successfully installed frontend addon',
            });
    }
}
/**
 * @function installCQAddons
 * @description handles installation of code quality addons into a STORM stack project
 * @param {Array<STORMCodeQualityAddon>} codeQualityAddons
 * @param {LoggerMode} loggerMode
 */
export async function installCQAddons(codeQualityAddons, loggerMode) {
    const verbose = loggerMode === 'verbose';
    for (const addon of codeQualityAddons) {
        if (verbose)
            ConsoleLogger.printCLIProcessInfoMessage(`Installing Code Quality addons`, addon);
        const addonResult = await scaffoldCodeQualityAddons(addon)();
        if (!addonResult.success) {
            if (verbose)
                ConsoleLogger.printCLIProcessErrorMessage(`${addonResult.message}`);
            process.exit(1);
        }
        if (verbose)
            ConsoleLogger.printCLIProcessSuccessMessage({
                message: 'Successfully installed code quality addon',
                detail: addon,
            });
    }
}
/**
 * @deprecated
 * @param {Array<STORMAddOn>} addOnOpts
 * @param {LoggerMode} loggerMode
 * @description Handles installation of all scaffold options
 */
export async function installScaffoldAddOns(addOnOpts, loggerMode) {
    const LocaleData = LocaleManager.getInstance().getLocaleData();
    const verbose = loggerMode === 'verbose';
    for (const addOn of addOnOpts) {
        if (verbose)
            ConsoleLogger.printLog(`${LocaleData.frontend.info.INSTALL_FE_ADDON}: ${addOn}`);
        const addOnFuncResult = await scaffoldAddOns(addOn)();
        if (!addOnFuncResult.success) {
            ConsoleLogger.printLog(`${LocaleData.frontend.error.INSTALL_FE_ADDON}: ${addOn} -> ${addOnFuncResult.message}`, 'error');
            process.exit(1);
        }
        if (verbose)
            ConsoleLogger.printLog(`${LocaleData.frontend.success.INSTALL_FE_ADDON}: ${addOn}`, 'success');
    }
}
//# sourceMappingURL=scaffoldAddOns.js.map